import {expect} from 'chai';
import {Whether, Match, Else} from '../src';
import ReactTestRenderer from 'react-test-renderer';
import ShallowRenderer from 'react-test-renderer/shallow';

const expectValue = (result, value) => {
    if (value === null) {
        expect(result).to.equal(null);
    }
    else {
        expect(result.props.children).to.equal(value);
    }
};

describe('react-whether', () => {
    it('should work with single matched if branch', () => {
        const renderer = new ShallowRenderer();
        renderer.render(
            <Whether matches={1 === 1}>
                <span>hit</span>
            </Whether>
        );
        const result = renderer.getRenderOutput();
        expectValue(result, 'hit');
    });

    it('should work with single non-matched if branch', () => {
        const renderer = new ShallowRenderer();
        renderer.render(
            <Whether matches={1 === 2}>
                <span>hit</span>
            </Whether>
        );
        const result = renderer.getRenderOutput();
        expectValue(result, null);
    });

    it('should render null when matches is not true', () => {
        const renderer = new ShallowRenderer();
        renderer.render(
            <Whether matches={1 === 2}>
                hit
            </Whether>
        );
        const result = renderer.getRenderOutput();
        expectValue(result, null);
    });

    it('should render null when no child', () => {
        const renderer = new ShallowRenderer();
        const result = renderer.getRenderOutput();
        expectValue(result, null);
    });

    it('should work with if/else pair', () => {
        const renderer = new ShallowRenderer();
        renderer.render(
            <Whether matches={1 === 2}>
                <span>not hit</span>
                <Else>
                    <span>hit</span>
                </Else>
            </Whether>
        );
        const result = renderer.getRenderOutput();
        expectValue(result, 'hit');
    });

    it('should render the first matched Match component', () => {
        const renderer = new ShallowRenderer();
        renderer.render(
            <Whether context={1}>
                <Match selector={n => n === 2}>
                    <span>not hit</span>
                </Match>
                <Match selector={n => n === 1}>
                    <span>hit</span>
                </Match>
                <Else>
                    <span>not hit</span>
                </Else>
            </Whether>
        );
        const result = renderer.getRenderOutput();
        expectValue(result, 'hit');
    });

    it('should render else when no Match component matches', () => {
        const renderer = new ShallowRenderer();
        renderer.render(
            <Whether context={1}>
                <Match selector={n => n === 2}>
                    <span>not hit</span>
                </Match>
                <Match selector={n => n === 3}>
                    <span>not hit</span>
                </Match>
                <Else>
                    <span>hit</span>
                </Else>
            </Whether>
        );
        const result = renderer.getRenderOutput();
        expectValue(result, 'hit');
    });

    it('should render null when no match', () => {
        const renderer = new ShallowRenderer();
        renderer.render(
            <Whether context={1}>
                <Match selector={n => n === 2}>
                    <span>not hit</span>
                </Match>
                <Match selector={n => n === 3}>
                    <span>hit</span>
                </Match>
            </Whether>
        );
        const result = renderer.getRenderOutput();
        expectValue(result, null);
    });

    it('should accept function as child', () => {
        const result = ReactTestRenderer.create(
            <Whether context={1}>
                <Match selector={n => n === 2}>
                    {() => <span>not hit</span>}
                </Match>
                <Match selector={n => n === 1}>
                    {() => <span>hit</span>}
                </Match>
            </Whether>
        );
        expect(result.toJSON().children).to.deep.equal(['hit']);
    });
});
